unit app_utils;

interface

uses Windows, SysUtils, Classes, ShellApi,
     obimp_dll, u_obimp_const, u_obimp_codes;

type
  TCmdAppMsgInfo = record
    SendTo: string;
    MsgText: string;
    MsgAck: Boolean;
  end;

  TCmdAppOtherInfo = record
    ShowHelp: Boolean;
    AuthReply: Word;
    NoReconnect: Boolean;
    StayConnected: Boolean;
    Interval: Cardinal;
  end;

  TAppCmdParams = record
    LoginInfo: TLoginInfo;
    ClientInfo: TClientInfo;
    StatusInfo: TStatusInfo;
    MsgInfo: TCmdAppMsgInfo;
    OtherInfo: TCmdAppOtherInfo;
  end;

  procedure Log(s: string);
  function  LowerUnicode(S: string): string;
  procedure GetCmdParams(var acp: TAppCmdParams);

const
  CR               = #13;
  LF               = #10;
  TAB              = #9;
  SPC              = ' ';
  CRLF             = CR + LF;

  HELP_INFO = '* --------------- Help -----------------' + CRLF + CRLF +
                SPC + 'Available command line parameters:' + CRLF +
                SPC + SPC + '/? or /help' + TAB + TAB + 'Show this help information' + CRLF + CRLF +
                SPC + SPC + '/id={user}' + TAB + TAB + 'Username/account' + CRLF +
                SPC + SPC + '/pwd={password}' + TAB + 'Username password' + CRLF +
                SPC + SPC + '/srv={server host}' + TAB + 'OBIMP server host or IPv4 address' + CRLF +
                SPC + SPC + '/port={server port}' + TAB + 'OBIMP server port (SSL is not supported)' + CRLF + CRLF +

                SPC + SPC + '/client={name}' + TAB + 'OBIMP client name, add double quote if includes space char, (default=ConsoleApp)' + CRLF +
                SPC + SPC + '/type={client type}' + TAB + 'OBIMP client type: user (default), bot, service' + CRLF +
                SPC + SPC + '/ver={client version}' + TAB + 'OBIMP client version, dots separated numbers: 1.0.0.0 (default)' + CRLF +
                SPC + SPC + '/status={status}' + TAB + 'Available status values: online (default), ffc, home, work, lunch, away, na, occ, dnd, invisible, ifa' + CRLF +
                SPC + SPC + '/authreply={reply}' + TAB + 'Reply on authorization requests: grant, deny' + CRLF + CRLF +

                SPC + SPC + '/fileto={file path}' + TAB + 'File path with list of recipients accounts (UTF-8, one per line)' + CRLF +
                SPC + SPC + '/filetext={file path}' + TAB + 'File path with message text (UTF-8, max=16384 chars)' + CRLF +
                SPC + SPC + '/interval={msec}' + TAB + 'Set timer interval for message sending in milliseconds (min=250, max=60000, default=500)' + CRLF +
                SPC + SPC + '/ackreq' + TAB + TAB + 'Set message acknowledge flag to receive delivery notifications' + CRLF +
                SPC + SPC + '/stay' + TAB + TAB + TAB + 'Do not quit after sending messages, stay connected' + CRLF +
                SPC + SPC + '/norecon' + TAB + TAB + 'Do not make reconnect attempt if connection failed or aborted' + CRLF + CRLF +

                CRLF + SPC + SPC + 'Example: ' + CRLF +
                SPC + SPC + 'ObimpCmd.exe /id=admin /pwd=123 /srv=192.168.1.1 /port=7023 /client="My App" /status=ffc /fileto=acc.txt /filetext=msg.txt';

implementation

const
  CMD_LINE_HELP         = '/help';
  CMD_LINE_Q            = '/?';
  CMD_LINE_ID           = '/id=';
  CMD_LINE_SRV          = '/srv=';
  CMD_LINE_PORT         = '/port=';
  CMD_LINE_PWD          = '/pwd=';
  CMD_LINE_CLIENT       = '/client=';
  CMD_LINE_TYPE         = '/type=';
  CMD_LINE_VER          = '/ver=';
  CMD_LINE_STATUS       = '/status=';
  CMD_LINE_AUTHREP      = '/authreply=';
  CMD_LINE_FILETO       = '/fileto=';
  CMD_LINE_FILETEXT     = '/filetext=';
  CMD_LINE_INTERVAL     = '/interval=';
  CMD_LINE_ACKREQ       = '/ackreq';
  CMD_LINE_STAY         = '/stay';
  CMD_LINE_NORECON      = '/norecon';

  CMD_LINE_ST_FFC       = 'ffc';
  CMD_LINE_ST_ATHOME    = 'home';
  CMD_LINE_ST_ATWORK    = 'work';
  CMD_LINE_ST_LUNCH     = 'lunch';
  CMD_LINE_ST_AWAY      = 'away';
  CMD_LINE_ST_NA        = 'na';
  CMD_LINE_ST_OCCUPIED  = 'occ';
  CMD_LINE_ST_DND       = 'dnd';
  CMD_LINE_ST_ONLINE    = 'online';
  CMD_LINE_ST_INVIS     = 'invisible';
  CMD_LINE_ST_IFA       = 'ifa';

  CMD_LINE_CT_USER      = 'user';
  CMD_LINE_CT_BOT       = 'bot';
  CMD_LINE_CT_SERVICE   = 'service';

  CMD_LINE_AR_GRANT     = 'grant';
  CMD_LINE_AR_DENY      = 'deny';

{***************************************************************}
procedure Log(s: string);
var NumWritten: Cardinal;
begin
  NumWritten := 0;
  s := TimeToStr(Now) + ' ' + s + CRLF;
  //supports unicode
  WriteConsole(GetStdHandle(STD_OUTPUT_HANDLE), PChar(s), Length(s), NumWritten, nil);
end;

{***************************************************************}
function ToLowerTmp(const S: string): string;
begin
  Result := S;
  if Result <> '' then
  begin
    UniqueString(Result);
    if LCMapString(GetThreadLocale, LCMAP_LOWERCASE, PChar(S), Length(S), PChar(Result), Length(Result)) = 0 then
      RaiseLastOSError;
  end;
end;

{***************************************************************}
function LowerUnicode(S: string): string;
begin
  if (Length(S) > 0) then
    Result := ToLowerTmp(S)
  else
    Result := S;
end;

{***************************************************************}
function LoadTextFromFile(const FileName: string): string;
var List: TStringList;
begin
  Result := '';

  if not FileExists(FileName) then
    Exit;

  List := TStringList.Create;
  try
    List.LoadFromFile(FileName, TEncoding.UTF8);
    Result := Trim(List.Text);
  finally
    List.Free;
  end;
end;

{***************************************************************}
function StatusStrToCode(Cmd: string): Cardinal;
begin
  Result := PRES_STATUS_ONLINE;

  if WideSameText(Cmd, CMD_LINE_ST_FFC) then
    Result := PRES_STATUS_FREE_FOR_CHAT
  else
  if WideSameText(Cmd, CMD_LINE_ST_ATHOME) then
    Result := PRES_STATUS_AT_HOME
  else
  if WideSameText(Cmd, CMD_LINE_ST_ATWORK) then
    Result := PRES_STATUS_AT_WORK
  else
  if WideSameText(Cmd, CMD_LINE_ST_LUNCH) then
    Result := PRES_STATUS_LUNCH
  else
  if WideSameText(Cmd, CMD_LINE_ST_AWAY) then
    Result := PRES_STATUS_AWAY
  else
  if WideSameText(Cmd, CMD_LINE_ST_NA) then
    Result := PRES_STATUS_NOT_AVAILABLE
  else
  if WideSameText(Cmd, CMD_LINE_ST_OCCUPIED) then
    Result := PRES_STATUS_OCCUPIED
  else
  if WideSameText(Cmd, CMD_LINE_ST_DND) then
    Result := PRES_STATUS_DO_NOT_DISTURB
  else
  if WideSameText(Cmd, CMD_LINE_ST_ONLINE) then
    Result := PRES_STATUS_ONLINE
  else
  if WideSameText(Cmd, CMD_LINE_ST_INVIS) then
    Result := PRES_STATUS_INVISIBLE
  else
  if WideSameText(Cmd, CMD_LINE_ST_IFA) then
    Result := PRES_STATUS_INVISIBLE_FOR_ALL;
end;

{***************************************************************}
function ClientTypeStrToCode(Cmd: string): Word;
begin
  Result := CLIENT_TYPE_USER;

  if WideSameText(Cmd, CMD_LINE_CT_USER) then
    Result := CLIENT_TYPE_USER
  else
  if WideSameText(Cmd, CMD_LINE_CT_BOT) then
    Result := CLIENT_TYPE_BOT
  else
  if WideSameText(Cmd, CMD_LINE_CT_SERVICE) then
    Result := CLIENT_TYPE_SERVICE;
end;

{***************************************************************}
function CountChars(const s: string; const c: Char): Integer;
var i: Integer;
begin
  Result := 0;
  for i := 1 to Length(s) do
    if (s[i] = C) then
      Inc(Result);
end;

{***************************************************************}
procedure ClientVerStrToVer(s: string; var ci: TClientInfo);
  function GetNextVal: Word;
  var i: Integer;
  begin
    i := Pos('.', s);
    Result := StrToIntDef(Trim(Copy(s, 1, i-1)), 0);
    Delete(s, 1, i);
  end;
begin
  s := s + '.';
  if (CountChars(s, '.') = 4) then
  begin
    ci.ClientVerMaj   := GetNextVal;
    ci.ClientVerMin   := GetNextVal;
    ci.ClientVerRel   := GetNextVal;
    ci.ClientVerBuild := GetNextVal;
  end;
end;

{***************************************************************}
function AuthReplyStrToCode(Cmd: string): Word;
begin
  Result := 0;

  if WideSameText(Cmd, CMD_LINE_AR_GRANT) then
    Result := AUTH_REPLY_GRANTED
  else
  if WideSameText(Cmd, CMD_LINE_AR_DENY) then
    Result := AUTH_REPLY_DENIED;
end;

{***************************************************************}
procedure GetCmdParams(var acp: TAppCmdParams);
type TCmdArgs = array[0..0] of PWideChar;
var args: ^TCmdArgs;
    i, iCount: integer;
    s: string;
begin
  Finalize(acp);
  ZeroMemory(@acp, SizeOf(TAppCmdParams));

  //set default params
  with acp.ClientInfo do
  begin
    ClientName     := 'ConsoleApp';
    ClientVerMaj   := 1;
    ClientVerMin   := 0;
    ClientVerRel   := 0;
    ClientVerBuild := 0;
    ClientType     := CLIENT_TYPE_USER;
    ClientLang     := LANGUAGE_CODE_RUSSIAN;
  end;

  acp.OtherInfo.Interval := 500;

  //parse cmd line
  args := Pointer(CommandLineToArgvW(GetCommandLineW, iCount));

  if (args <> nil) then
  begin
    for i := 0 to iCount-1 do
    begin
      s := Trim(LowerUnicode(args[i]));

      //skip exe path
      if (i = 0) then
      begin
        if (iCount = 1) then
          acp.OtherInfo.ShowHelp := True;

        Continue;
      end;

      if (Pos(CMD_LINE_HELP, s) = 1) or (Pos(CMD_LINE_Q, s) = 1) then
      begin
        acp.OtherInfo.ShowHelp := True;
        Break;
      end
      else
      if (Pos(CMD_LINE_ID, s) = 1) then
      begin
        Delete(s, 1, Length(CMD_LINE_ID));
        acp.LoginInfo.Account := Trim(s);
      end
      else
      if (Pos(CMD_LINE_SRV, s) = 1) then
      begin
        Delete(s, 1, Length(CMD_LINE_SRV));
        acp.LoginInfo.Host := Trim(s);
      end
      else
      if (Pos(CMD_LINE_PORT, s) = 1) then
      begin
        Delete(s, 1, Length(CMD_LINE_PORT));
        acp.LoginInfo.Port := StrToIntDef(Trim(s), 0);
      end
      else
      if (Pos(CMD_LINE_PWD, s) = 1) then
      begin
        s := args[i]; //case sensetive
        Delete(s, 1, Length(CMD_LINE_PWD));
        acp.LoginInfo.Password := Trim(s);
      end
      else
      if (Pos(CMD_LINE_CLIENT, s) = 1) then
      begin
        s := args[i]; //case sensetive
        Delete(s, 1, Length(CMD_LINE_CLIENT));
        acp.ClientInfo.ClientName := Copy(Trim(s), 1, 64);
      end
      else
      if (Pos(CMD_LINE_TYPE, s) = 1) then
      begin
        Delete(s, 1, Length(CMD_LINE_TYPE));
        acp.ClientInfo.ClientType := ClientTypeStrToCode(Trim(s));
      end
      else
      if (Pos(CMD_LINE_VER, s) = 1) then
      begin
        Delete(s, 1, Length(CMD_LINE_VER));
        ClientVerStrToVer(Trim(s), acp.ClientInfo);
      end
      else
      if (Pos(CMD_LINE_STATUS, s) = 1) then
      begin
        Delete(s, 1, Length(CMD_LINE_STATUS));
        acp.StatusInfo.Status := StatusStrToCode(Trim(s));
      end
      else
      if (Pos(CMD_LINE_AUTHREP, s) = 1) then
      begin
        Delete(s, 1, Length(CMD_LINE_AUTHREP));
        acp.OtherInfo.AuthReply := AuthReplyStrToCode(Trim(s));
      end
      else
      if (Pos(CMD_LINE_FILETO, s) = 1) then
      begin
        s := args[i]; //case sensetive
        Delete(s, 1, Length(CMD_LINE_FILETO));
        acp.MsgInfo.SendTo := LoadTextFromFile(Trim(s));
      end
      else
      if (Pos(CMD_LINE_FILETEXT, s) = 1) then
      begin
        s := args[i]; //case sensetive
        Delete(s, 1, Length(CMD_LINE_FILETEXT));
        acp.MsgInfo.MsgText := Copy(LoadTextFromFile(Trim(s)), 1, 16384);
      end
      else
      if (Pos(CMD_LINE_INTERVAL, s) = 1) then
      begin
        Delete(s, 1, Length(CMD_LINE_INTERVAL));
        acp.OtherInfo.Interval := StrToIntDef(Trim(s), 0);
      end
      else
      if (CMD_LINE_ACKREQ = s) then
        acp.MsgInfo.MsgAck := True
      else
      if (CMD_LINE_STAY = s) then
        acp.OtherInfo.StayConnected := True
      else
      if (CMD_LINE_NORECON = s) then
        acp.OtherInfo.NoReconnect := True;
    end;//for

    LocalFree(THandle(args));
  end;
end;

end.
