//************************************************************
// OBIMP protocol dll types
//
// Records are not packed, but quad-word aligned {$ALIGN 8}.
//
// Types description:
//  - DWord, Integer types size = 4 bytes
//  - Word type size = 2 bytes
//  - Byte, Boolean types size = 1 byte
//  - Int64 type size = 8 bytes
//  - WideString type is OLE BSTR compatible type
//************************************************************

unit obimp_dll;

interface

uses Windows;

const
  //dll name
  OBIMP_DLL_FILE_NAME = 'mobimp.dll';

  //dll export function name
  DLL_EXP_FNAME_CREATE_OBIMP = 'CreateObimp';

  //protocol interface version
  PROTO_INTF_VERSION = 1003;

  //protocol interface functions result values
  FUNC_OPER_SUCCESS = 0;
  FUNC_OPER_ERROR   = 1;

  //protocol login failed code values
  LOGIN_FAILED_INVALID_LOGIN     = 1;
  LOGIN_FAILED_TEMP_UNAVAILABLE  = 2;
  LOGIN_FAILED_TOO_MANY_CLIENTS  = 3;
  LOGIN_FAILED_OTHER_SRV_ERROR   = 4;

  //protocol bye reason codes
  //BYE_REASON_... can be found in u_obimp_const.pas

type
  //login information
  TLoginInfo = record
    Host     : WideString;
    Port     : DWord;
    Account  : WideString;
    Password : WideString;
  end;
  pLoginInfo = ^TLoginInfo;

  //client information
  TClientInfo = record
    ClientName     : WideString;
    ClientVerMaj   : Word;
    ClientVerMin   : Word;
    ClientVerRel   : Word;
    ClientVerBuild : Word;
    ClientType     : Word;
    ClientLang     : Word;
  end;
  pClientInfo = ^TClientInfo;

  //status information
  TStatusInfo = record
    Status         : DWord;
    StatusName     : WideString;
  end;
  pStatusInfo = ^TStatusInfo;

  //received message information
  TRcvdMsgInfo = record
    Account       : WideString;
    MsgUniqID     : DWord;
    MsgType       : DWord;
    MsgText       : WideString;
    AckRequired   : Boolean;
    IsOfflineMsg  : Boolean;
    OfflineDT     : Int64;     //unix datetime format
    IsSystemMsg   : Boolean;
    SysMsgPos     : Byte;
    TranspID      : DWord;
  end;
  pRcvdMsgInfo = ^TRcvdMsgInfo;

  //message acknowledge
  TMsgAck = record
    Account       : WideString;
    MsgUniqID     : DWord;
    TranspID      : DWord;
  end;
  pMsgAck = ^TMsgAck;

  //send message information
  TSendMsgInfo = record
    Account       : WideString;
    MsgUniqID     : DWord;
    MsgText       : WideString;
    AckRequired   : Boolean;
    TranspID      : DWord;
  end;
  pSendMsgInfo = ^TSendMsgInfo;

  //received authorization request
  TRcvdAuthReq = record
    Account       : WideString;
    ReasonText    : WideString;
    IsOfflineMsg  : Boolean;
    OfflineDT     : Int64;     //unix datetime format
    TranspID      : DWord;
  end;
  pRcvdAuthReq = ^TRcvdAuthReq;

  //send authorization request
  TSendAuthReq = record
    Account       : WideString;
    ReasonText    : WideString;
    TranspID      : DWord;
  end;
  pSendAuthReq = ^TSendAuthReq;

  //received authorization reply
  TRcvdAuthReply = record
    Account       : WideString;
    ReplyCode     : Word;
    IsOfflineMsg  : Boolean;
    OfflineDT     : Int64;     //unix datetime format
    TranspID      : DWord;
  end;
  pRcvdAuthReply = ^TRcvdAuthReply;

  //send authorization reply
  TSendAuthReply = record
    Account       : WideString;
    ReplyCode     : Word;
    TranspID      : DWord;
  end;
  pSendAuthReply = ^TSendAuthReply;

  //received authorization revoke
  TRcvdAuthRevoke = record
    Account       : WideString;
    ReasonText    : WideString;
    IsOfflineMsg  : Boolean;
    OfflineDT     : Int64;     //unix datetime format
    TranspID      : DWord;
  end;
  pRcvdAuthRevoke = ^TRcvdAuthRevoke;

  //send authorization revoke
  TSendAuthRevoke = record
    Account       : WideString;
    ReasonText    : WideString;
    TranspID      : DWord;
  end;
  pSendAuthRevoke = ^TSendAuthRevoke;


  //protocol interface
  IObimpProto = interface
    function  GetProtoIntfVer(): DWord; stdcall;
    function  Login(LoginInfo: pLoginInfo; ClientInfo: pClientInfo; StatusInfo: pStatusInfo): integer; stdcall;
    function  Logoff(): integer; stdcall;
    function  ChangeStatus(StatusInfo: pStatusInfo): integer; stdcall;
    function  SendMsg(SendMsgInfo: pSendMsgInfo): integer; stdcall;
    function  SendMsgAck(MsgAck: pMsgAck): integer; stdcall;
    function  SendAuthReq(AuthReq: pSendAuthReq): integer; stdcall;
    function  SendAuthReply(AuthReply: pSendAuthReply): integer; stdcall;
    function  SendAuthRevoke(AuthRevoke: pSendAuthRevoke): integer; stdcall;
  end;

  //protocol events interface
  IObimpEvents = interface
    function  GetProtoIntfVer(): DWord; stdcall;
    procedure ConnectionFailed(); stdcall;
    procedure LoginSuccess(); stdcall;
    procedure LoginFailed(LoginFailedCode: Word); stdcall;
    procedure ServerBye(ByeReasonCode: Word); stdcall;
    procedure MsgReceived(RcvdMsgInfo: pRcvdMsgInfo); stdcall;
    procedure MsgAckRcvd(MsgAck: pMsgAck); stdcall;
    procedure AuthReqRcvd(AuthReq: pRcvdAuthReq); stdcall;
    procedure AuthReplyRcvd(AuthReply: pRcvdAuthReply); stdcall;
    procedure AuthRevokeRcvd(AuthRevoke: pRcvdAuthRevoke); stdcall;
  end;


implementation

end.
